@@ -17,7 +17,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
-/* $Id: bitbang.h 1294 2014-03-12 23:03:18Z joerg_wunsch $ */
+/* $Id: bitbang.h 1301 2014-05-16 15:52:25Z joerg_wunsch $ */
 
 #ifndef bitbang_h
 #define bitbang_h
@@ -31,7 +31,7 @@ int bitbang_getpin(int fd, int pin);
 int bitbang_highpulsepin(int fd, int pin);
 void bitbang_delay(unsigned int us);
 
-void bitbang_check_prerequisites(PROGRAMMER *pgm);
+int bitbang_check_prerequisites(PROGRAMMER *pgm);
 
 int  bitbang_rdy_led        (PROGRAMMER * pgm, int value);
 int  bitbang_err_led        (PROGRAMMER * pgm, int value);
