@@ -16,7 +16,7 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: jtagmkI.c 1294 2014-03-12 23:03:18Z joerg_wunsch $ */
+/* $Id: jtagmkI.c 1345 2014-11-23 20:57:17Z joerg_wunsch $ */
 
 /*
  * avrdude interface for Atmel JTAG ICE (mkI) programmer
@@ -33,12 +33,11 @@
 #include <time.h>
 
 #include "avrdude.h"
-#include "avr.h"
+#include "libavrdude.h"
+
 #include "crc16.h"
-#include "pgm.h"
 #include "jtagmkI.h"
 #include "jtagmkI_private.h"
-#include "serial.h"
 
 /*
  * Private data for this programmer.
@@ -112,9 +111,8 @@ static int jtagmkI_resync(PROGRAMMER *pgm, int maxtries, int signon);
 static void jtagmkI_setup(PROGRAMMER * pgm)
 {
   if ((pgm->cookie = malloc(sizeof(struct pdata))) == 0) {
-    fprintf(stderr,
-	    "%s: jtagmkI_setup(): Out of memory allocating private data\n",
-	    progname);
+    avrdude_message(MSG_INFO, "%s: jtagmkI_setup(): Out of memory allocating private data\n",
+                    progname);
     exit(1);
   }
   memset(pgm->cookie, 0, sizeof(struct pdata));
@@ -146,10 +144,10 @@ static void jtagmkI_prmsg(PROGRAMMER * pgm, unsigned char * data, size_t len)
   int i;
 
   if (verbose >= 4) {
-    fprintf(stderr, "Raw message:\n");
+    avrdude_message(MSG_TRACE, "Raw message:\n");
 
     for (i = 0; i < len; i++) {
-      fprintf(stderr, "0x%02x ", data[i]);
+      avrdude_message(MSG_TRACE, "0x%02x ", data[i]);
       if (i % 16 == 15)
 	putc('\n', stderr);
       else
@@ -161,34 +159,34 @@ static void jtagmkI_prmsg(PROGRAMMER * pgm, unsigned char * data, size_t len)
 
   switch (data[0]) {
   case RESP_OK:
-    fprintf(stderr, "OK\n");
+    avrdude_message(MSG_INFO, "OK\n");
     break;
 
   case RESP_FAILED:
-    fprintf(stderr, "FAILED\n");
+    avrdude_message(MSG_INFO, "FAILED\n");
     break;
 
   case RESP_BREAK:
-    fprintf(stderr, "breakpoint hit\n");
+    avrdude_message(MSG_INFO, "breakpoint hit\n");
     break;
 
   case RESP_INFO:
-    fprintf(stderr, "IDR dirty\n");
+    avrdude_message(MSG_INFO, "IDR dirty\n");
     break;
 
   case RESP_SYNC_ERROR:
-    fprintf(stderr, "Synchronization lost\n");
+    avrdude_message(MSG_INFO, "Synchronization lost\n");
     break;
 
   case RESP_SLEEP:
-    fprintf(stderr, "sleep instruction hit\n");
+    avrdude_message(MSG_INFO, "sleep instruction hit\n");
     break;
 
   case RESP_POWER:
-    fprintf(stderr, "target power lost\n");
+    avrdude_message(MSG_INFO, "target power lost\n");
 
   default:
-    fprintf(stderr, "unknown message 0x%02x\n", data[0]);
+    avrdude_message(MSG_INFO, "unknown message 0x%02x\n", data[0]);
   }
 
   putc('\n', stderr);
@@ -199,13 +197,12 @@ static int jtagmkI_send(PROGRAMMER * pgm, unsigned char * data, size_t len)
 {
   unsigned char *buf;
 
-  if (verbose >= 3)
-    fprintf(stderr, "\n%s: jtagmkI_send(): sending %u bytes\n",
+  avrdude_message(MSG_DEBUG, "\n%s: jtagmkI_send(): sending %u bytes\n",
 	    progname, (unsigned int)len);
 
   if ((buf = malloc(len + 2)) == NULL)
     {
-      fprintf(stderr, "%s: jtagmkI_send(): out of memory",
+      avrdude_message(MSG_INFO, "%s: jtagmkI_send(): out of memory",
 	      progname);
       exit(1);
     }
@@ -215,10 +212,9 @@ static int jtagmkI_send(PROGRAMMER * pgm, unsigned char * data, size_t len)
   buf[len + 1] = ' ';		/* EOP */
 
   if (serial_send(&pgm->fd, buf, len + 2) != 0) {
-    fprintf(stderr,
-	    "%s: jtagmkI_send(): failed to send command to serial port\n",
-	    progname);
-    exit(1);
+    avrdude_message(MSG_INFO, "%s: jtagmkI_send(): failed to send command to serial port\n",
+                    progname);
+    return -1;
   }
 
   free(buf);
@@ -226,18 +222,18 @@ static int jtagmkI_send(PROGRAMMER * pgm, unsigned char * data, size_t len)
   return 0;
 }
 
-static void jtagmkI_recv(PROGRAMMER * pgm, unsigned char * buf, size_t len)
+static int jtagmkI_recv(PROGRAMMER * pgm, unsigned char * buf, size_t len)
 {
   if (serial_recv(&pgm->fd, buf, len) != 0) {
-    fprintf(stderr,
-	    "\n%s: jtagmkI_recv(): failed to send command to serial port\n",
-	    progname);
-    exit(1);
+    avrdude_message(MSG_INFO, "\n%s: jtagmkI_recv(): failed to send command to serial port\n",
+                    progname);
+    return -1;
   }
   if (verbose >= 3) {
     putc('\n', stderr);
     jtagmkI_prmsg(pgm, buf, len);
   }
+  return 0;
 }
 
 
@@ -255,8 +251,7 @@ static int jtagmkI_resync(PROGRAMMER * pgm, int maxtries, int signon)
 
   serial_recv_timeout = 200;
 
-  if (verbose >= 3)
-    fprintf(stderr, "%s: jtagmkI_resync()\n", progname);
+  avrdude_message(MSG_TRACE, "%s: jtagmkI_resync()\n", progname);
 
   jtagmkI_drain(pgm, 0);
 
@@ -264,20 +259,17 @@ static int jtagmkI_resync(PROGRAMMER * pgm, int maxtries, int signon)
 
     /* Get the sign-on information. */
     buf[0] = CMD_GET_SYNC;
-    if (verbose >= 2)
-      fprintf(stderr, "%s: jtagmkI_resync(): Sending sync command: ",
+    avrdude_message(MSG_NOTICE2, "%s: jtagmkI_resync(): Sending sync command: ",
 	      progname);
 
     if (serial_send(&pgm->fd, buf, 1) != 0) {
-      fprintf(stderr,
-	      "\n%s: jtagmkI_resync(): failed to send command to serial port\n",
-	      progname);
+      avrdude_message(MSG_INFO, "\n%s: jtagmkI_resync(): failed to send command to serial port\n",
+                      progname);
       serial_recv_timeout = otimeout;
       return -1;
     }
     if (serial_recv(&pgm->fd, resp, 1) == 0 && resp[0] == RESP_OK) {
-      if (verbose >= 2)
-	fprintf(stderr, "got RESP_OK\n");
+      avrdude_message(MSG_NOTICE2, "got RESP_OK\n");
       break;
     }
 
@@ -296,30 +288,25 @@ static int jtagmkI_resync(PROGRAMMER * pgm, int maxtries, int signon)
       buf[1] = 'E';
       buf[2] = ' ';
       buf[3] = ' ';
-      if (verbose >= 2)
-	fprintf(stderr, "%s: jtagmkI_resync(): Sending sign-on command: ",
+      avrdude_message(MSG_NOTICE2, "%s: jtagmkI_resync(): Sending sign-on command: ",
 		progname);
 
       if (serial_send(&pgm->fd, buf, 4) != 0) {
-	fprintf(stderr,
-		"\n%s: jtagmkI_resync(): failed to send command to serial port\n",
-		progname);
+	avrdude_message(MSG_INFO, "\n%s: jtagmkI_resync(): failed to send command to serial port\n",
+                        progname);
 	serial_recv_timeout = otimeout;
 	return -1;
       }
       if (serial_recv(&pgm->fd, resp, 9) == 0 && resp[0] == RESP_OK) {
-	if (verbose >= 2)
-	  fprintf(stderr, "got RESP_OK\n");
+        avrdude_message(MSG_NOTICE2, "got RESP_OK\n");
 	break;
       }
     }
   }
   if (tries >= maxtries) {
-    if (verbose >= 2)
-      fprintf(stderr,
-	      "%s: jtagmkI_resync(): "
-	      "timeout/error communicating with programmer\n",
-	      progname);
+    avrdude_message(MSG_NOTICE2, "%s: jtagmkI_resync(): "
+                      "timeout/error communicating with programmer\n",
+                      progname);
     serial_recv_timeout = otimeout;
     return -1;
   }
@@ -339,16 +326,16 @@ static int jtagmkI_getsync(PROGRAMMER * pgm)
 
   jtagmkI_drain(pgm, 0);
 
-  if (verbose >= 2)
-    fprintf(stderr, "%s: jtagmkI_getsync(): Sending sign-on command: ",
+  avrdude_message(MSG_NOTICE2, "%s: jtagmkI_getsync(): Sending sign-on command: ",
 	    progname);
 
   buf[0] = CMD_GET_SIGNON;
   jtagmkI_send(pgm, buf, 1);
-  jtagmkI_recv(pgm, resp, 9);
+  if (jtagmkI_recv(pgm, resp, 9) < 0)
+    return -1;
   if (verbose >= 2) {
     resp[8] = '\0';
-    fprintf(stderr, "got %s\n", resp + 1);
+    avrdude_message(MSG_NOTICE2, "got %s\n", resp + 1);
   }
 
   return 0;
@@ -362,22 +349,21 @@ static int jtagmkI_chip_erase(PROGRAMMER * pgm, AVRPART * p)
   unsigned char buf[1], resp[2];
 
   buf[0] = CMD_CHIP_ERASE;
-  if (verbose >= 2)
-    fprintf(stderr, "%s: jtagmkI_chip_erase(): Sending chip erase command: ",
+  avrdude_message(MSG_NOTICE2, "%s: jtagmkI_chip_erase(): Sending chip erase command: ",
 	    progname);
   jtagmkI_send(pgm, buf, 1);
-  jtagmkI_recv(pgm, resp, 2);
+  if (jtagmkI_recv(pgm, resp, 2) < 0)
+    return -1;
   if (resp[0] != RESP_OK) {
     if (verbose >= 2)
       putc('\n', stderr);
-    fprintf(stderr,
-	    "%s: jtagmkI_chip_erase(): "
-	    "timeout/error communicating with programmer (resp %c)\n",
-	    progname, resp[0]);
+    avrdude_message(MSG_INFO, "%s: jtagmkI_chip_erase(): "
+                    "timeout/error communicating with programmer (resp %c)\n",
+                    progname, resp[0]);
     return -1;
   } else {
     if (verbose == 2)
-      fprintf(stderr, "OK\n");
+      avrdude_message(MSG_NOTICE2, "OK\n");
   }
 
   pgm->initialize(pgm, p);
@@ -410,23 +396,22 @@ static void jtagmkI_set_devdescr(PROGRAMMER * pgm, AVRPART * p)
     }
   }
 
-  if (verbose >= 2)
-    fprintf(stderr, "%s: jtagmkI_set_devdescr(): "
+  avrdude_message(MSG_NOTICE2, "%s: jtagmkI_set_devdescr(): "
 	    "Sending set device descriptor command: ",
 	    progname);
   jtagmkI_send(pgm, (unsigned char *)&sendbuf, sizeof(sendbuf));
 
-  jtagmkI_recv(pgm, resp, 2);
+  if (jtagmkI_recv(pgm, resp, 2) < 0)
+    return;
   if (resp[0] != RESP_OK) {
     if (verbose >= 2)
       putc('\n', stderr);
-    fprintf(stderr,
-	    "%s: jtagmkI_set_devdescr(): "
-	    "timeout/error communicating with programmer (resp %c)\n",
-	    progname, resp[0]);
+    avrdude_message(MSG_INFO, "%s: jtagmkI_set_devdescr(): "
+                    "timeout/error communicating with programmer (resp %c)\n",
+                    progname, resp[0]);
   } else {
     if (verbose == 2)
-      fprintf(stderr, "OK\n");
+      avrdude_message(MSG_NOTICE2, "OK\n");
   }
 }
 
@@ -438,23 +423,22 @@ static int jtagmkI_reset(PROGRAMMER * pgm)
   unsigned char buf[1], resp[2];
 
   buf[0] = CMD_RESET;
-  if (verbose >= 2)
-    fprintf(stderr, "%s: jtagmkI_reset(): Sending reset command: ",
+  avrdude_message(MSG_NOTICE2, "%s: jtagmkI_reset(): Sending reset command: ",
 	    progname);
   jtagmkI_send(pgm, buf, 1);
 
-  jtagmkI_recv(pgm, resp, 2);
+  if (jtagmkI_recv(pgm, resp, 2) < 0)
+    return -1;
   if (resp[0] != RESP_OK) {
     if (verbose >= 2)
       putc('\n', stderr);
-    fprintf(stderr,
-	    "%s: jtagmkI_reset(): "
-	    "timeout/error communicating with programmer (resp %c)\n",
-	    progname, resp[0]);
+    avrdude_message(MSG_INFO, "%s: jtagmkI_reset(): "
+                    "timeout/error communicating with programmer (resp %c)\n",
+                    progname, resp[0]);
     return -1;
   } else {
     if (verbose == 2)
-      fprintf(stderr, "OK\n");
+      avrdude_message(MSG_NOTICE2, "OK\n");
   }
 
   return 0;
@@ -474,24 +458,23 @@ static int jtagmkI_program_enable(PROGRAMMER * pgm)
     return 0;
 
   buf[0] = CMD_ENTER_PROGMODE;
-  if (verbose >= 2)
-    fprintf(stderr, "%s: jtagmkI_program_enable(): "
+  avrdude_message(MSG_NOTICE2, "%s: jtagmkI_program_enable(): "
 	    "Sending enter progmode command: ",
 	    progname);
   jtagmkI_send(pgm, buf, 1);
 
-  jtagmkI_recv(pgm, resp, 2);
+  if (jtagmkI_recv(pgm, resp, 2) < 0)
+    return -1;
   if (resp[0] != RESP_OK) {
     if (verbose >= 2)
       putc('\n', stderr);
-    fprintf(stderr,
-	    "%s: jtagmkI_program_enable(): "
-	    "timeout/error communicating with programmer (resp %c)\n",
-	    progname, resp[0]);
+    avrdude_message(MSG_INFO, "%s: jtagmkI_program_enable(): "
+                    "timeout/error communicating with programmer (resp %c)\n",
+                    progname, resp[0]);
     return -1;
   } else {
     if (verbose == 2)
-      fprintf(stderr, "OK\n");
+      avrdude_message(MSG_NOTICE2, "OK\n");
   }
 
   PDATA(pgm)->prog_enabled = 1;
@@ -508,24 +491,23 @@ static int jtagmkI_program_disable(PROGRAMMER * pgm)
 
   if (pgm->fd.ifd != -1) {
     buf[0] = CMD_LEAVE_PROGMODE;
-    if (verbose >= 2)
-      fprintf(stderr, "%s: jtagmkI_program_disable(): "
+    avrdude_message(MSG_NOTICE2, "%s: jtagmkI_program_disable(): "
               "Sending leave progmode command: ",
               progname);
     jtagmkI_send(pgm, buf, 1);
 
-    jtagmkI_recv(pgm, resp, 2);
+    if (jtagmkI_recv(pgm, resp, 2) < 0)
+      return -1;
     if (resp[0] != RESP_OK) {
       if (verbose >= 2)
         putc('\n', stderr);
-      fprintf(stderr,
-              "%s: jtagmkI_program_disable(): "
-              "timeout/error communicating with programmer (resp %c)\n",
-              progname, resp[0]);
+      avrdude_message(MSG_INFO, "%s: jtagmkI_program_disable(): "
+                      "timeout/error communicating with programmer (resp %c)\n",
+                      progname, resp[0]);
       return -1;
     } else {
       if (verbose == 2)
-        fprintf(stderr, "OK\n");
+        avrdude_message(MSG_NOTICE2, "OK\n");
     }
   }
   PDATA(pgm)->prog_enabled = 0;
@@ -554,7 +536,7 @@ static int jtagmkI_initialize(PROGRAMMER * pgm, AVRPART * p)
   unsigned char b;
 
   if (!(p->flags & AVRPART_HAS_JTAG)) {
-    fprintf(stderr, "%s: jtagmkI_initialize(): part %s has no JTAG interface\n",
+    avrdude_message(MSG_INFO, "%s: jtagmkI_initialize(): part %s has no JTAG interface\n",
 	    progname, p->desc);
     return -1;
   }
@@ -563,11 +545,10 @@ static int jtagmkI_initialize(PROGRAMMER * pgm, AVRPART * p)
 
   if ((serdev->flags & SERDEV_FL_CANSETSPEED) && PDATA(pgm)->initial_baudrate != pgm->baudrate) {
     if ((b = jtagmkI_get_baud(pgm->baudrate)) == 0) {
-      fprintf(stderr, "%s: jtagmkI_initialize(): unsupported baudrate %d\n",
+      avrdude_message(MSG_INFO, "%s: jtagmkI_initialize(): unsupported baudrate %d\n",
               progname, pgm->baudrate);
     } else {
-      if (verbose >= 2)
-        fprintf(stderr, "%s: jtagmkI_initialize(): "
+      avrdude_message(MSG_NOTICE2, "%s: jtagmkI_initialize(): "
 	      "trying to set baudrate to %d\n",
                 progname, pgm->baudrate);
       if (jtagmkI_setparm(pgm, PARM_BITRATE, b) == 0) {
@@ -578,8 +559,7 @@ static int jtagmkI_initialize(PROGRAMMER * pgm, AVRPART * p)
   }
 
   if (pgm->bitclock != 0.0) {
-    if (verbose >= 2)
-      fprintf(stderr, "%s: jtagmkI_initialize(): "
+    avrdude_message(MSG_NOTICE2, "%s: jtagmkI_initialize(): "
 	      "trying to set JTAG clock period to %.1f us\n",
 	      progname, pgm->bitclock);
     if (jtagmkI_set_sck_period(pgm, pgm->bitclock) != 0)
@@ -588,17 +568,17 @@ static int jtagmkI_initialize(PROGRAMMER * pgm, AVRPART * p)
 
   cmd[0] = CMD_STOP;
   jtagmkI_send(pgm, cmd, 1);
-  jtagmkI_recv(pgm, resp, 5);
+  if (jtagmkI_recv(pgm, resp, 5) < 0)
+    return -1;
   if (resp[0] != RESP_OK) {
     if (verbose >= 2)
       putc('\n', stderr);
-    fprintf(stderr,
-	    "%s: jtagmkI_initialize(): "
-	    "timeout/error communicating with programmer (resp %c)\n",
-	    progname, resp[0]);
+    avrdude_message(MSG_INFO, "%s: jtagmkI_initialize(): "
+                    "timeout/error communicating with programmer (resp %c)\n",
+                    progname, resp[0]);
   } else {
     if (verbose == 2)
-      fprintf(stderr, "OK\n");
+      avrdude_message(MSG_NOTICE2, "OK\n");
   }
 
   /*
@@ -613,12 +593,12 @@ static int jtagmkI_initialize(PROGRAMMER * pgm, AVRPART * p)
   free(PDATA(pgm)->flash_pagecache);
   free(PDATA(pgm)->eeprom_pagecache);
   if ((PDATA(pgm)->flash_pagecache = malloc(PDATA(pgm)->flash_pagesize)) == NULL) {
-    fprintf(stderr, "%s: jtagmkI_initialize(): Out of memory\n",
+    avrdude_message(MSG_INFO, "%s: jtagmkI_initialize(): Out of memory\n",
 	    progname);
     return -1;
   }
   if ((PDATA(pgm)->eeprom_pagecache = malloc(PDATA(pgm)->eeprom_pagesize)) == NULL) {
-    fprintf(stderr, "%s: jtagmkI_initialize(): Out of memory\n",
+    avrdude_message(MSG_INFO, "%s: jtagmkI_initialize(): Out of memory\n",
 	    progname);
     free(PDATA(pgm)->flash_pagecache);
     return -1;
@@ -632,10 +612,9 @@ static int jtagmkI_initialize(PROGRAMMER * pgm, AVRPART * p)
   if (jtagmkI_read_byte(pgm, p, &hfuse, 1, &b) < 0)
     return -1;
   if ((b & OCDEN) != 0)
-    fprintf(stderr,
-	    "%s: jtagmkI_initialize(): warning: OCDEN fuse not programmed, "
-	    "single-byte EEPROM updates not possible\n",
-	    progname);
+    avrdude_message(MSG_INFO, "%s: jtagmkI_initialize(): warning: OCDEN fuse not programmed, "
+                    "single-byte EEPROM updates not possible\n",
+                    progname);
 
   return 0;
 }
@@ -662,8 +641,7 @@ static int jtagmkI_open(PROGRAMMER * pgm, char * port)
 {
   size_t i;
 
-  if (verbose >= 2)
-    fprintf(stderr, "%s: jtagmkI_open()\n", progname);
+  avrdude_message(MSG_NOTICE2, "%s: jtagmkI_open()\n", progname);
 
   strcpy(pgm->port, port);
   PDATA(pgm)->initial_baudrate = -1L;
@@ -671,10 +649,8 @@ static int jtagmkI_open(PROGRAMMER * pgm, char * port)
   for (i = 0; i < sizeof(baudtab) / sizeof(baudtab[0]); i++) {
     union pinfo pinfo;
     pinfo.baud = baudtab[i].baud;
-    if (verbose >= 2)
-      fprintf(stderr,
-              "%s: jtagmkI_open(): trying to sync at baud rate %ld:\n",
-              progname, pinfo.baud);
+    avrdude_message(MSG_NOTICE2, "%s: jtagmkI_open(): trying to sync at baud rate %ld:\n",
+                      progname, pinfo.baud);
     if (serial_open(port, pinfo, &pgm->fd)==-1) {
       return -1;
     }
@@ -686,17 +662,15 @@ static int jtagmkI_open(PROGRAMMER * pgm, char * port)
 
     if (jtagmkI_getsync(pgm) == 0) {
       PDATA(pgm)->initial_baudrate = baudtab[i].baud;
-      if (verbose >= 2)
-        fprintf(stderr, "%s: jtagmkI_open(): succeeded\n", progname);
+      avrdude_message(MSG_NOTICE2, "%s: jtagmkI_open(): succeeded\n", progname);
       return 0;
     }
 
     serial_close(&pgm->fd);
   }
 
-  fprintf(stderr,
-          "%s: jtagmkI_open(): failed to synchronize to ICE\n",
-          progname);
+  avrdude_message(MSG_INFO, "%s: jtagmkI_open(): failed to synchronize to ICE\n",
+                  progname);
   pgm->fd.ifd = -1;
 
   return -1;
@@ -707,8 +681,7 @@ static void jtagmkI_close(PROGRAMMER * pgm)
 {
   unsigned char b;
 
-  if (verbose >= 2)
-    fprintf(stderr, "%s: jtagmkI_close()\n", progname);
+  avrdude_message(MSG_NOTICE2, "%s: jtagmkI_close()\n", progname);
 
   /*
    * Revert baud rate to what it used to be when we started.  This
@@ -717,11 +690,10 @@ static void jtagmkI_close(PROGRAMMER * pgm)
    */
   if ((serdev->flags & SERDEV_FL_CANSETSPEED) && PDATA(pgm)->initial_baudrate != pgm->baudrate) {
     if ((b = jtagmkI_get_baud(PDATA(pgm)->initial_baudrate)) == 0) {
-      fprintf(stderr, "%s: jtagmkI_close(): unsupported baudrate %d\n",
+      avrdude_message(MSG_INFO, "%s: jtagmkI_close(): unsupported baudrate %d\n",
               progname, PDATA(pgm)->initial_baudrate);
     } else {
-      if (verbose >= 2)
-        fprintf(stderr, "%s: jtagmkI_close(): "
+      avrdude_message(MSG_NOTICE2, "%s: jtagmkI_close(): "
                 "trying to set baudrate to %d\n",
                 progname, PDATA(pgm)->initial_baudrate);
       if (jtagmkI_setparm(pgm, PARM_BITRATE, b) == 0) {
@@ -750,8 +722,7 @@ static int jtagmkI_paged_write(PROGRAMMER * pgm, AVRPART * p, AVRMEM * m,
   long otimeout = serial_recv_timeout;
 #define MAXTRIES 3
 
-  if (verbose >= 2)
-    fprintf(stderr, "%s: jtagmkI_paged_write(.., %s, %d, %d)\n",
+  avrdude_message(MSG_NOTICE2, "%s: jtagmkI_paged_write(.., %s, %d, %d)\n",
 	    progname, m->desc, page_size, n_bytes);
 
   if (jtagmkI_program_enable(pgm) < 0)
@@ -760,13 +731,13 @@ static int jtagmkI_paged_write(PROGRAMMER * pgm, AVRPART * p, AVRMEM * m,
   if (page_size == 0) page_size = 256;
 
   if (page_size > 256) {
-    fprintf(stderr, "%s: jtagmkI_paged_write(): page size %d too large\n",
+    avrdude_message(MSG_INFO, "%s: jtagmkI_paged_write(): page size %d too large\n",
 	    progname, page_size);
     return -1;
   }
 
   if ((datacmd = malloc(page_size + 1)) == NULL) {
-    fprintf(stderr, "%s: jtagmkI_paged_write(): Out of memory\n",
+    avrdude_message(MSG_INFO, "%s: jtagmkI_paged_write(): Out of memory\n",
 	    progname);
     return -1;
   }
@@ -790,18 +761,16 @@ static int jtagmkI_paged_write(PROGRAMMER * pgm, AVRPART * p, AVRMEM * m,
     again:
 
     if (tries != 0 && jtagmkI_resync(pgm, 2000, 0) < 0) {
-      fprintf(stderr,
-	      "%s: jtagmkI_paged_write(): sync loss, retries exhausted\n",
-	      progname);
+      avrdude_message(MSG_INFO, "%s: jtagmkI_paged_write(): sync loss, retries exhausted\n",
+                      progname);
       return -1;
     }
 
-    if ((n_bytes-addr) < page_size)
-      block_size = n_bytes - addr;
+    if (n_bytes < page_size)
+      block_size = n_bytes;
     else
       block_size = page_size;
-    if (verbose >= 3)
-      fprintf(stderr, "%s: jtagmkI_paged_write(): "
+    avrdude_message(MSG_DEBUG, "%s: jtagmkI_paged_write(): "
 	      "block_size at addr %d is %d\n",
 	      progname, addr, block_size);
 
@@ -815,28 +784,27 @@ static int jtagmkI_paged_write(PROGRAMMER * pgm, AVRPART * p, AVRMEM * m,
       u32_to_b3(cmd + 3, addr);
     }
 
-    if (verbose >= 2)
-      fprintf(stderr, "%s: jtagmkI_paged_write(): "
+    avrdude_message(MSG_NOTICE2, "%s: jtagmkI_paged_write(): "
 	      "Sending write memory command: ",
 	      progname);
 
     /* First part, send the write command. */
     jtagmkI_send(pgm, cmd, 6);
-    jtagmkI_recv(pgm, resp, 1);
+    if (jtagmkI_recv(pgm, resp, 1) < 0)
+      return -1;
     if (resp[0] != RESP_OK) {
       if (verbose >= 2)
         putc('\n', stderr);
-      fprintf(stderr,
-              "%s: jtagmkI_paged_write(): "
-              "timeout/error communicating with programmer (resp %c)\n",
-              progname, resp[0]);
+      avrdude_message(MSG_INFO, "%s: jtagmkI_paged_write(): "
+                      "timeout/error communicating with programmer (resp %c)\n",
+                      progname, resp[0]);
       if (tries++ < MAXTRIES)
 	goto again;
       serial_recv_timeout = otimeout;
       return -1;
     } else {
       if (verbose == 2)
-        fprintf(stderr, "OK\n");
+        avrdude_message(MSG_NOTICE2, "OK\n");
     }
 
     /*
@@ -851,21 +819,21 @@ static int jtagmkI_paged_write(PROGRAMMER * pgm, AVRPART * p, AVRMEM * m,
 
     /* Second, send the data command. */
     jtagmkI_send(pgm, datacmd, send_size + 1);
-    jtagmkI_recv(pgm, resp, 2);
+    if (jtagmkI_recv(pgm, resp, 2) < 0)
+      return -1;
     if (resp[1] != RESP_OK) {
       if (verbose >= 2)
         putc('\n', stderr);
-      fprintf(stderr,
-              "%s: jtagmkI_paged_write(): "
-              "timeout/error communicating with programmer (resp %c)\n",
-              progname, resp[0]);
+      avrdude_message(MSG_INFO, "%s: jtagmkI_paged_write(): "
+                      "timeout/error communicating with programmer (resp %c)\n",
+                      progname, resp[0]);
       if (tries++ < MAXTRIES)
 	goto again;
       serial_recv_timeout = otimeout;
       return -1;
     } else {
       if (verbose == 2)
-        fprintf(stderr, "OK\n");
+        avrdude_message(MSG_NOTICE2, "OK\n");
     }
   }
 
@@ -886,8 +854,7 @@ static int jtagmkI_paged_load(PROGRAMMER * pgm, AVRPART * p, AVRMEM * m,
   long otimeout = serial_recv_timeout;
 #define MAXTRIES 3
 
-  if (verbose >= 2)
-    fprintf(stderr, "%s: jtagmkI_paged_load(.., %s, %d, %d)\n",
+  avrdude_message(MSG_NOTICE2, "%s: jtagmkI_paged_load(.., %s, %d, %d)\n",
 	    progname, m->desc, page_size, n_bytes);
 
   if (jtagmkI_program_enable(pgm) < 0)
@@ -904,7 +871,7 @@ static int jtagmkI_paged_load(PROGRAMMER * pgm, AVRPART * p, AVRMEM * m,
   }
 
   if (page_size > (is_flash? 512: 256)) {
-    fprintf(stderr, "%s: jtagmkI_paged_load(): page size %d too large\n",
+    avrdude_message(MSG_INFO, "%s: jtagmkI_paged_load(): page size %d too large\n",
 	    progname, page_size);
     return -1;
   }
@@ -914,18 +881,16 @@ static int jtagmkI_paged_load(PROGRAMMER * pgm, AVRPART * p, AVRMEM * m,
     tries = 0;
     again:
     if (tries != 0 && jtagmkI_resync(pgm, 2000, 0) < 0) {
-      fprintf(stderr,
-	      "%s: jtagmkI_paged_load(): sync loss, retries exhausted\n",
-	      progname);
+      avrdude_message(MSG_INFO, "%s: jtagmkI_paged_load(): sync loss, retries exhausted\n",
+                      progname);
       return -1;
     }
 
-    if ((n_bytes-addr) < page_size)
-      block_size = n_bytes - addr;
+    if (n_bytes < page_size)
+      block_size = n_bytes;
     else
       block_size = page_size;
-    if (verbose >= 3)
-      fprintf(stderr, "%s: jtagmkI_paged_load(): "
+    avrdude_message(MSG_DEBUG, "%s: jtagmkI_paged_load(): "
 	      "block_size at addr %d is %d\n",
 	      progname, addr, block_size);
 
@@ -939,20 +904,19 @@ static int jtagmkI_paged_load(PROGRAMMER * pgm, AVRPART * p, AVRMEM * m,
       u32_to_b3(cmd + 3, addr);
     }
 
-    if (verbose >= 2)
-      fprintf(stderr, "%s: jtagmkI_paged_load(): Sending read memory command: ",
+    avrdude_message(MSG_NOTICE2, "%s: jtagmkI_paged_load(): Sending read memory command: ",
 	      progname);
 
     jtagmkI_send(pgm, cmd, 6);
-    jtagmkI_recv(pgm, resp, read_size + 3);
+    if (jtagmkI_recv(pgm, resp, read_size + 3) < 0)
+      return -1;
 
     if (resp[read_size + 3 - 1] != RESP_OK) {
       if (verbose >= 2)
         putc('\n', stderr);
-      fprintf(stderr,
-              "%s: jtagmkI_paged_load(): "
-              "timeout/error communicating with programmer (resp %c)\n",
-              progname, resp[read_size + 3 - 1]);
+      avrdude_message(MSG_INFO, "%s: jtagmkI_paged_load(): "
+                      "timeout/error communicating with programmer (resp %c)\n",
+                      progname, resp[read_size + 3 - 1]);
       if (tries++ < MAXTRIES)
 	goto again;
 
@@ -960,7 +924,7 @@ static int jtagmkI_paged_load(PROGRAMMER * pgm, AVRPART * p, AVRMEM * m,
       return -1;
     } else {
       if (verbose == 2)
-        fprintf(stderr, "OK\n");
+        avrdude_message(MSG_NOTICE2, "OK\n");
     }
 
     memcpy(m->buf + addr, resp + 1, block_size);
@@ -981,8 +945,7 @@ static int jtagmkI_read_byte(PROGRAMMER * pgm, AVRPART * p, AVRMEM * mem,
   int respsize = 3 + 1;
   int is_flash = 0;
 
-  if (verbose >= 2)
-    fprintf(stderr, "%s: jtagmkI_read_byte(.., %s, 0x%lx, ...)\n",
+  avrdude_message(MSG_NOTICE2, "%s: jtagmkI_read_byte(.., %s, 0x%lx, ...)\n",
 	    progname, mem->desc, addr);
 
   if (jtagmkI_program_enable(pgm) < 0)
@@ -1060,19 +1023,19 @@ static int jtagmkI_read_byte(PROGRAMMER * pgm, AVRPART * p, AVRMEM * mem,
   }
 
   jtagmkI_send(pgm, cmd, 6);
-  jtagmkI_recv(pgm, resp, respsize);
+  if (jtagmkI_recv(pgm, resp, respsize) < 0)
+    return -1;
 
   if (resp[respsize - 1] != RESP_OK) {
     if (verbose >= 2)
       putc('\n', stderr);
-    fprintf(stderr,
-	    "%s: jtagmkI_read_byte(): "
-	    "timeout/error communicating with programmer (resp %c)\n",
-	    progname, resp[respsize - 1]);
-    exit(1);
+    avrdude_message(MSG_INFO, "%s: jtagmkI_read_byte(): "
+                    "timeout/error communicating with programmer (resp %c)\n",
+                    progname, resp[respsize - 1]);
+    return -1;
   } else {
     if (verbose == 2)
-      fprintf(stderr, "OK\n");
+      avrdude_message(MSG_NOTICE2, "OK\n");
   }
 
   if (pagesize) {
@@ -1095,8 +1058,7 @@ static int jtagmkI_write_byte(PROGRAMMER * pgm, AVRPART * p, AVRMEM * mem,
   unsigned char resp[1], writedata;
   int len, need_progmode = 1;
 
-  if (verbose >= 2)
-    fprintf(stderr, "%s: jtagmkI_write_byte(.., %s, 0x%lx, ...)\n",
+  avrdude_message(MSG_NOTICE2, "%s: jtagmkI_write_byte(.., %s, 0x%lx, ...)\n",
 	    progname, mem->desc, addr);
 
   writedata = data;
@@ -1146,18 +1108,18 @@ static int jtagmkI_write_byte(PROGRAMMER * pgm, AVRPART * p, AVRMEM * mem,
   }
   /* First part, send the write command. */
   jtagmkI_send(pgm, cmd, 6);
-  jtagmkI_recv(pgm, resp, 1);
+  if (jtagmkI_recv(pgm, resp, 1) < 0)
+    return -1;
   if (resp[0] != RESP_OK) {
     if (verbose >= 2)
       putc('\n', stderr);
-    fprintf(stderr,
-	    "%s: jtagmkI_write_byte(): "
-	    "timeout/error communicating with programmer (resp %c)\n",
-	    progname, resp[0]);
+    avrdude_message(MSG_INFO, "%s: jtagmkI_write_byte(): "
+                    "timeout/error communicating with programmer (resp %c)\n",
+                    progname, resp[0]);
     return -1;
   } else {
     if (verbose == 2)
-      fprintf(stderr, "OK\n");
+      avrdude_message(MSG_NOTICE2, "OK\n");
   }
 
   /* Now, send the data buffer. */
@@ -1176,18 +1138,18 @@ static int jtagmkI_write_byte(PROGRAMMER * pgm, AVRPART * p, AVRMEM * mem,
     datacmd[1] = writedata;
   }
   jtagmkI_send(pgm, datacmd, len);
-  jtagmkI_recv(pgm, resp, 1);
+  if (jtagmkI_recv(pgm, resp, 1) < 0)
+    return -1;
   if (resp[0] != RESP_OK) {
     if (verbose >= 2)
       putc('\n', stderr);
-    fprintf(stderr,
-	    "%s: jtagmkI_write_byte(): "
-	    "timeout/error communicating with programmer (resp %c)\n",
-	    progname, resp[0]);
+    avrdude_message(MSG_INFO, "%s: jtagmkI_write_byte(): "
+                    "timeout/error communicating with programmer (resp %c)\n",
+                    progname, resp[0]);
     return -1;
   } else {
     if (verbose == 2)
-      fprintf(stderr, "OK\n");
+      avrdude_message(MSG_NOTICE2, "OK\n");
   }
 
   return 0;
@@ -1231,37 +1193,35 @@ static int jtagmkI_getparm(PROGRAMMER * pgm, unsigned char parm,
 {
   unsigned char buf[2], resp[3];
 
-  if (verbose >= 2)
-    fprintf(stderr, "%s: jtagmkI_getparm()\n", progname);
+  avrdude_message(MSG_NOTICE2, "%s: jtagmkI_getparm()\n", progname);
 
   buf[0] = CMD_GET_PARAM;
   buf[1] = parm;
   if (verbose >= 2)
-    fprintf(stderr, "%s: jtagmkI_getparm(): "
+    avrdude_message(MSG_NOTICE2, "%s: jtagmkI_getparm(): "
 	    "Sending get parameter command (parm 0x%02x): ",
 	    progname, parm);
   jtagmkI_send(pgm, buf, 2);
 
-  jtagmkI_recv(pgm, resp, 3);
+  if (jtagmkI_recv(pgm, resp, 3) < 0)
+    return -1;
   if (resp[0] != RESP_OK) {
     if (verbose >= 2)
       putc('\n', stderr);
-    fprintf(stderr,
-	    "%s: jtagmkI_getparm(): "
-	    "timeout/error communicating with programmer (resp %c)\n",
-	    progname, resp[0]);
+    avrdude_message(MSG_INFO, "%s: jtagmkI_getparm(): "
+                    "timeout/error communicating with programmer (resp %c)\n",
+                    progname, resp[0]);
     return -1;
   } else if (resp[2] != RESP_OK) {
     if (verbose >= 2)
       putc('\n', stderr);
-    fprintf(stderr,
-	    "%s: jtagmkI_getparm(): "
-	    "unknown parameter 0x%02x\n",
-	    progname, parm);
+    avrdude_message(MSG_INFO, "%s: jtagmkI_getparm(): "
+                    "unknown parameter 0x%02x\n",
+                    progname, parm);
     return -1;
   } else {
     if (verbose == 2)
-      fprintf(stderr, "OK, value 0x%02x\n", resp[1]);
+      avrdude_message(MSG_NOTICE2, "OK, value 0x%02x\n", resp[1]);
   }
 
   *value = resp[1];
@@ -1277,29 +1237,27 @@ static int jtagmkI_setparm(PROGRAMMER * pgm, unsigned char parm,
 {
   unsigned char buf[3], resp[2];
 
-  if (verbose >= 2)
-    fprintf(stderr, "%s: jtagmkI_setparm()\n", progname);
+  avrdude_message(MSG_NOTICE2, "%s: jtagmkI_setparm()\n", progname);
 
   buf[0] = CMD_SET_PARAM;
   buf[1] = parm;
   buf[2] = value;
-  if (verbose >= 2)
-    fprintf(stderr, "%s: jtagmkI_setparm(): "
+  avrdude_message(MSG_NOTICE2, "%s: jtagmkI_setparm(): "
 	    "Sending set parameter command (parm 0x%02x): ",
 	    progname, parm);
   jtagmkI_send(pgm, buf, 3);
-  jtagmkI_recv(pgm, resp, 2);
+  if (jtagmkI_recv(pgm, resp, 2) < 0)
+    return -1;
   if (resp[0] != RESP_OK) {
     if (verbose >= 2)
       putc('\n', stderr);
-    fprintf(stderr,
-	    "%s: jtagmkI_setparm(): "
-	    "timeout/error communicating with programmer (resp %c)\n",
-	    progname, resp[0]);
+    avrdude_message(MSG_INFO, "%s: jtagmkI_setparm(): "
+                    "timeout/error communicating with programmer (resp %c)\n",
+                    progname, resp[0]);
     return -1;
   } else {
     if (verbose == 2)
-      fprintf(stderr, "OK\n");
+      avrdude_message(MSG_NOTICE2, "OK\n");
   }
 
   return 0;
@@ -1315,8 +1273,8 @@ static void jtagmkI_display(PROGRAMMER * pgm, const char * p)
       jtagmkI_getparm(pgm, PARM_SW_VERSION, &fw) < 0)
     return;
 
-  fprintf(stderr, "%sICE hardware version: 0x%02x\n", p, hw);
-  fprintf(stderr, "%sICE firmware version: 0x%02x\n", p, fw);
+  avrdude_message(MSG_INFO, "%sICE hardware version: 0x%02x\n", p, hw);
+  avrdude_message(MSG_INFO, "%sICE firmware version: 0x%02x\n", p, fw);
 
   jtagmkI_print_parms1(pgm, p);
 
@@ -1360,9 +1318,9 @@ static void jtagmkI_print_parms1(PROGRAMMER * pgm, const char * p)
     clk = 1e6;
   }
 
-  fprintf(stderr, "%sVtarget         : %.1f V\n", p,
+  avrdude_message(MSG_INFO, "%sVtarget         : %.1f V\n", p,
 	  6.25 * (unsigned)vtarget / 255.0);
-  fprintf(stderr, "%sJTAG clock      : %s (%.1f us)\n", p, clkstr,
+  avrdude_message(MSG_INFO, "%sJTAG clock      : %s (%.1f us)\n", p, clkstr,
 	  1.0e6 / clk);
 
   return;
