@@ -16,7 +16,7 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: ppi.c 1294 2014-03-12 23:03:18Z joerg_wunsch $ */
+/* $Id: ppi.c 1321 2014-06-13 20:07:40Z awachtler $ */
 
 
 #if !defined(WIN32NATIVE)
@@ -41,9 +41,8 @@
 #endif
 
 #include "avrdude.h"
-#include "avr.h"
-#include "pindefs.h"
-#include "pgm.h"
+#include "libavrdude.h"
+
 #include "ppi.h"
 
 enum {
@@ -69,7 +68,7 @@ static int ppi_shadow_access(union filedescriptor *fdp, int reg,
       shadow_num = 2;
       break;
     default:
-      fprintf(stderr, "%s: avr_set(): invalid register=%d\n",
+      avrdude_message(MSG_INFO, "%s: avr_set(): invalid register=%d\n",
               progname, reg);
       return -1;
       break;
@@ -206,7 +205,7 @@ void ppi_open(char * port, union filedescriptor *fdp)
 
   fd = open(port, O_RDWR);
   if (fd < 0) {
-    fprintf(stderr, "%s: can't open device \"%s\": %s\n",
+    avrdude_message(MSG_INFO, "%s: can't open device \"%s\": %s\n",
               progname, port, strerror(errno));
     fdp->ifd = -1;
     return;
