@@ -17,7 +17,7 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: ser_posix.c 1294 2014-03-12 23:03:18Z joerg_wunsch $ */
+/* $Id: ser_posix.c 1351 2014-11-25 21:33:22Z joerg_wunsch $ */
 
 /*
  * Posix serial interface for avrdude.
@@ -37,13 +37,16 @@
 #include <sys/socket.h>
 #include <netdb.h>
 #include <netinet/in.h>
+#ifdef __linux__
+#include <linux/serial.h>
+#endif
 
 #include <fcntl.h>
 #include <termios.h>
 #include <unistd.h>
 
 #include "avrdude.h"
-#include "serial.h"
+#include "libavrdude.h"
 
 long serial_recv_timeout = 5000; /* ms */
 
@@ -52,8 +55,13 @@ struct baud_mapping {
   speed_t speed;
 };
 
-/* There are a lot more baud rates we could handle, but what's the point? */
+static struct termios original_termios;
+static int saved_original_termios;
 
+#if !defined __linux__
+/* For linux this mapping is no longer needed.
+ * (OSX and *BSD do not need this mapping either because for them,
+ * Bxxx is the same as xxx.) */
 static struct baud_mapping baud_lookup_table [] = {
   { 1200,   B1200 },
   { 2400,   B2400 },
@@ -73,8 +81,6 @@ static struct baud_mapping baud_lookup_table [] = {
   { 0,      0 }                 /* Terminator. */
 };
 
-static struct termios original_termios;
-static int saved_original_termios;
 
 static speed_t serial_baud_lookup(long baud)
 {
@@ -90,18 +96,24 @@ static speed_t serial_baud_lookup(long baud)
    * If a non-standard BAUD rate is used, issue
    * a warning (if we are verbose) and return the raw rate
    */
-  if (verbose > 0)
-      fprintf(stderr, "%s: serial_baud_lookup(): Using non-standard baud rate: %ld",
+  avrdude_message(MSG_NOTICE, "%s: serial_baud_lookup(): Using non-standard baud rate: %ld",
               progname, baud);
 
   return baud;
 }
+#endif
 
 static int ser_setspeed(union filedescriptor *fd, long baud)
 {
   int rc;
   struct termios termios;
+#if defined __linux__
+  /* for linux no conversion is needed*/
+  speed_t speed = baud;
+#else
+  /* converting the baud rate to the bit set needed by posix way*/
   speed_t speed = serial_baud_lookup (baud);
+#endif
   
   if (!isatty(fd->ifd))
     return -ENOTTY;
@@ -111,7 +123,7 @@ static int ser_setspeed(union filedescriptor *fd, long baud)
    */
   rc = tcgetattr(fd->ifd, &termios);
   if (rc < 0) {
-    fprintf(stderr, "%s: ser_setspeed(): tcgetattr() failed",
+    avrdude_message(MSG_INFO, "%s: ser_setspeed(): tcgetattr() failed",
             progname);
     return -errno;
   }
@@ -129,16 +141,79 @@ static int ser_setspeed(union filedescriptor *fd, long baud)
   termios.c_cflag = (CS8 | CREAD | CLOCAL);
   termios.c_cc[VMIN]  = 1;
   termios.c_cc[VTIME] = 0;
+#ifdef __linux__
+  /* Support for custom baud rate for linux is implemented by setting
+   * a dummy baud rate of 38400 and manupulating the custom divider of
+   * the serial interface*/
+  struct serial_struct  ss;
+  int ioret = ioctl(fd->ifd, TIOCGSERIAL, &ss);
+  if (ioret < 0){
+    avrdude_message(MSG_INFO,
+		    "%s: Cannot get serial port settings. ioctl returned %d\n",
+		    progname, ioret);
+    return -errno;
+  }
+  ss.flags = (ss.flags & ~ASYNC_SPD_MASK) | ASYNC_SPD_CUST;
+  ss.custom_divisor = (ss.baud_base + (speed / 2)) / speed;
+  unsigned int closestSpeed = ss.baud_base / ss.custom_divisor;
 
-  cfsetospeed(&termios, speed);
-  cfsetispeed(&termios, speed);
-
+  if (closestSpeed < speed * 98 / 100 || closestSpeed > speed * 102 / 100) {
+    avrdude_message(MSG_INFO,
+		    "%s: Cannot set serial port speed to %d. Closest possible is %d\n",
+		    progname, speed, closestSpeed);
+    return -errno;
+  }
+  ioret= ioctl(fd->ifd, TIOCSSERIAL, &ss);
+  if (ioret < 0){
+    avrdude_message(MSG_INFO,
+		    "%s: Cannot set serial port speed to %d. ioctl returned %d\n",
+		    progname, speed, ioret);
+    return -errno;
+  }
+  if (cfsetispeed(&termios, B38400) < 0){
+    avrdude_message(MSG_INFO,
+		    "%s: cfsetispeed: failed to set dummy baud\n",
+		    progname);
+    return -errno;
+  }
+  if (cfsetospeed(&termios, B38400) < 0){
+    avrdude_message(MSG_INFO,
+		    "%s: cfsetospeed: failed to set dummy baud\n",
+		    progname);
+    return -errno;
+  }
+#else  /* !linux */
+  if (cfsetospeed(&termios, speed) < 0){
+    avrdude_message(MSG_INFO,
+		    "%s: cfsetospeed: failed to set speed: %d\n",
+		    progname, speed);
+    return -errno;
+  }
+  if (cfsetispeed(&termios, speed) < 0){
+    avrdude_message(MSG_INFO,
+		    "%s: cfsetispeed: failed to set speed: %d\n",
+		    progname, speed);
+    return -errno;
+  }
+#endif	/* linux */
   rc = tcsetattr(fd->ifd, TCSANOW, &termios);
   if (rc < 0) {
-    fprintf(stderr, "%s: ser_setspeed(): tcsetattr() failed\n",
+    avrdude_message(MSG_INFO, "%s: ser_setspeed(): tcsetattr() failed\n",
             progname);
     return -errno;
   }
+#ifdef __linux__
+  /* a bit more linux specific stuff to set custom baud rates*/
+  if (ioctl(fd->ifd, TIOCGSERIAL, &ss) < 0){
+    avrdude_message(MSG_INFO, "%s: ioctl: failed to get port settins\n", progname);
+    return -errno;
+  }
+  ss.flags &= ~ASYNC_SPD_MASK;
+  if (ioctl(fd->ifd, TIOCSSERIAL, &ss) < 0){
+    avrdude_message(MSG_INFO, "%s: ioctl: failed to set port settins\n", progname);
+    return -errno;
+  }
+#endif
 
   /*
    * Everything is now set up for a local line without modem control
@@ -167,13 +242,13 @@ net_open(const char *port, union filedescriptor *fdp)
   struct hostent *hp;
 
   if ((hstr = strdup(port)) == NULL) {
-    fprintf(stderr, "%s: net_open(): Out of memory!\n",
+    avrdude_message(MSG_INFO, "%s: net_open(): Out of memory!\n",
 	    progname);
     return -1;
   }
 
   if (((pstr = strchr(hstr, ':')) == NULL) || (pstr == hstr)) {
-    fprintf(stderr, "%s: net_open(): Mangled host:port string \"%s\"\n",
+    avrdude_message(MSG_INFO, "%s: net_open(): Mangled host:port string \"%s\"\n",
 	    progname, hstr);
     free(hstr);
     return -1;
@@ -187,14 +262,14 @@ net_open(const char *port, union filedescriptor *fdp)
   pnum = strtoul(pstr, &end, 10);
 
   if ((*pstr == '\0') || (*end != '\0') || (pnum == 0) || (pnum > 65535)) {
-    fprintf(stderr, "%s: net_open(): Bad port number \"%s\"\n",
+    avrdude_message(MSG_INFO, "%s: net_open(): Bad port number \"%s\"\n",
 	    progname, pstr);
     free(hstr);
     return -1;
   }
 
   if ((hp = gethostbyname(hstr)) == NULL) {
-    fprintf(stderr, "%s: net_open(): unknown host \"%s\"\n",
+    avrdude_message(MSG_INFO, "%s: net_open(): unknown host \"%s\"\n",
 	    progname, hstr);
     free(hstr);
     return -1;
@@ -203,7 +278,7 @@ net_open(const char *port, union filedescriptor *fdp)
   free(hstr);
 
   if ((fd = socket(PF_INET, SOCK_STREAM, 0)) < 0) {
-    fprintf(stderr, "%s: net_open(): Cannot open socket: %s\n",
+    avrdude_message(MSG_INFO, "%s: net_open(): Cannot open socket: %s\n",
 	    progname, strerror(errno));
     return -1;
   }
@@ -214,7 +289,7 @@ net_open(const char *port, union filedescriptor *fdp)
   memcpy(&(sockaddr.sin_addr.s_addr), hp->h_addr, sizeof(struct in_addr));
 
   if (connect(fd, (struct sockaddr *)&sockaddr, sizeof(sockaddr))) {
-    fprintf(stderr, "%s: net_open(): Connect failed: %s\n",
+    avrdude_message(MSG_INFO, "%s: net_open(): Connect failed: %s\n",
 	    progname, strerror(errno));
     return -1;
   }
@@ -271,7 +346,7 @@ static int ser_open(char * port, union pinfo pinfo, union filedescriptor *fdp)
    */
   fd = open(port, O_RDWR | O_NOCTTY | O_NONBLOCK);
   if (fd < 0) {
-    fprintf(stderr, "%s: ser_open(): can't open device \"%s\": %s\n",
+    avrdude_message(MSG_INFO, "%s: ser_open(): can't open device \"%s\": %s\n",
             progname, port, strerror(errno));
     return -1;
   }
@@ -283,9 +358,8 @@ static int ser_open(char * port, union pinfo pinfo, union filedescriptor *fdp)
    */
   rc = ser_setspeed(fdp, pinfo.baud);
   if (rc) {
-    fprintf(stderr, 
-            "%s: ser_open(): can't set attributes for device \"%s\": %s\n",
-            progname, port, strerror(-rc));
+    avrdude_message(MSG_INFO, "%s: ser_open(): can't set attributes for device \"%s\": %s\n",
+                    progname, port, strerror(-rc));
     close(fd);
     return -1;
   }
@@ -301,9 +375,8 @@ static void ser_close(union filedescriptor *fd)
   if (saved_original_termios) {
     int rc = tcsetattr(fd->ifd, TCSANOW | TCSADRAIN, &original_termios);
     if (rc) {
-      fprintf(stderr, 
-              "%s: ser_close(): can't reset attributes for device: %s\n",
-              progname, strerror(errno));
+      avrdude_message(MSG_INFO, "%s: ser_close(): can't reset attributes for device: %s\n",
+                      progname, strerror(errno));
     }
     saved_original_termios = 0;
   }
@@ -312,10 +385,10 @@ static void ser_close(union filedescriptor *fd)
 }
 
 
-static int ser_send(union filedescriptor *fd, unsigned char * buf, size_t buflen)
+static int ser_send(union filedescriptor *fd, const unsigned char * buf, size_t buflen)
 {
   int rc;
-  unsigned char * p = buf;
+  const unsigned char * p = buf;
   size_t len = buflen;
 
   if (!len)
@@ -323,31 +396,31 @@ static int ser_send(union filedescriptor *fd, unsigned char * buf, size_t buflen
 
   if (verbose > 3)
   {
-      fprintf(stderr, "%s: Send: ", progname);
+      avrdude_message(MSG_TRACE, "%s: Send: ", progname);
 
       while (buflen) {
         unsigned char c = *buf;
         if (isprint(c)) {
-          fprintf(stderr, "%c ", c);
+          avrdude_message(MSG_TRACE, "%c ", c);
         }
         else {
-          fprintf(stderr, ". ");
+          avrdude_message(MSG_TRACE, ". ");
         }
-        fprintf(stderr, "[%02x] ", c);
+        avrdude_message(MSG_TRACE, "[%02x] ", c);
 
         buf++;
         buflen--;
       }
 
-      fprintf(stderr, "\n");
+      avrdude_message(MSG_TRACE, "\n");
   }
 
   while (len) {
     rc = write(fd->ifd, p, (len > 1024) ? 1024 : len);
     if (rc < 0) {
-      fprintf(stderr, "%s: ser_send(): write error: %s\n",
+      avrdude_message(MSG_INFO, "%s: ser_send(): write error: %s\n",
               progname, strerror(errno));
-      exit(1);
+      return -1;
     }
     p += rc;
     len -= rc;
@@ -377,31 +450,28 @@ static int ser_recv(union filedescriptor *fd, unsigned char * buf, size_t buflen
 
     nfds = select(fd->ifd + 1, &rfds, NULL, NULL, &to2);
     if (nfds == 0) {
-      if (verbose > 1)
-	fprintf(stderr,
-		"%s: ser_recv(): programmer is not responding\n",
-		progname);
+      avrdude_message(MSG_NOTICE2, "%s: ser_recv(): programmer is not responding\n",
+                        progname);
       return -1;
     }
     else if (nfds == -1) {
       if (errno == EINTR || errno == EAGAIN) {
-	fprintf(stderr,
-		"%s: ser_recv(): programmer is not responding,reselecting\n",
-		progname);
+	avrdude_message(MSG_INFO, "%s: ser_recv(): programmer is not responding,reselecting\n",
+                        progname);
         goto reselect;
       }
       else {
-        fprintf(stderr, "%s: ser_recv(): select(): %s\n",
+        avrdude_message(MSG_INFO, "%s: ser_recv(): select(): %s\n",
                 progname, strerror(errno));
-        exit(1);
+        return -1;
       }
     }
 
     rc = read(fd->ifd, p, (buflen - len > 1024) ? 1024 : buflen - len);
     if (rc < 0) {
-      fprintf(stderr, "%s: ser_recv(): read error: %s\n",
+      avrdude_message(MSG_INFO, "%s: ser_recv(): read error: %s\n",
               progname, strerror(errno));
-      exit(1);
+      return -1;
     }
     p += rc;
     len += rc;
@@ -411,22 +481,22 @@ static int ser_recv(union filedescriptor *fd, unsigned char * buf, size_t buflen
 
   if (verbose > 3)
   {
-      fprintf(stderr, "%s: Recv: ", progname);
+      avrdude_message(MSG_TRACE, "%s: Recv: ", progname);
 
       while (len) {
         unsigned char c = *p;
         if (isprint(c)) {
-          fprintf(stderr, "%c ", c);
+          avrdude_message(MSG_TRACE, "%c ", c);
         }
         else {
-          fprintf(stderr, ". ");
+          avrdude_message(MSG_TRACE, ". ");
         }
-        fprintf(stderr, "[%02x] ", c);
+        avrdude_message(MSG_TRACE, "[%02x] ", c);
 
         p++;
         len--;
       }
-      fprintf(stderr, "\n");
+      avrdude_message(MSG_TRACE, "\n");
   }
 
   return 0;
@@ -445,7 +515,7 @@ static int ser_drain(union filedescriptor *fd, int display)
   timeout.tv_usec = 250000;
 
   if (display) {
-    fprintf(stderr, "drain>");
+    avrdude_message(MSG_INFO, "drain>");
   }
 
   while (1) {
@@ -456,7 +526,7 @@ static int ser_drain(union filedescriptor *fd, int display)
     nfds = select(fd->ifd + 1, &rfds, NULL, NULL, &timeout);
     if (nfds == 0) {
       if (display) {
-        fprintf(stderr, "<drain\n");
+        avrdude_message(MSG_INFO, "<drain\n");
       }
       
       break;
@@ -466,20 +536,20 @@ static int ser_drain(union filedescriptor *fd, int display)
         goto reselect;
       }
       else {
-        fprintf(stderr, "%s: ser_drain(): select(): %s\n",
+        avrdude_message(MSG_INFO, "%s: ser_drain(): select(): %s\n",
                 progname, strerror(errno));
-        exit(1);
+        return -1;
       }
     }
 
     rc = read(fd->ifd, &buf, 1);
     if (rc < 0) {
-      fprintf(stderr, "%s: ser_drain(): read error: %s\n",
+      avrdude_message(MSG_INFO, "%s: ser_drain(): read error: %s\n",
               progname, strerror(errno));
-      exit(1);
+      return -1;
     }
     if (display) {
-      fprintf(stderr, "%02x ", buf);
+      avrdude_message(MSG_INFO, "%02x ", buf);
     }
   }
 
